﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using GuasuoDLL;
using JmpRfidLp;

namespace DLLMain
{
    public partial class Form1 : Form
    {
        GuasuoDLL.RFIDClass RfidDll = new GuasuoDLL.RFIDClass();
        private Reader reader = null;
        private DeviceStatus deviceStatus = DeviceStatus.未连接;
        private List<string> epcList = new List<string>();
        public Form1()
        {
            InitializeComponent();

        }

        private void button_3des_Click(object sender, EventArgs e)
        {
            string key_data = textBox_key.Text;
            string cmd_data = textBox_cmd.Text;
            string Uii_str = textBox_UII.Text;
            string opere_time = DateTime.Now.ToString("yyyyMMddHHmmss");
            try
            {

                string rfid_jiami = RfidDll.rfid_lock_jiami(Uii_str, key_data, cmd_data);
                textBox_3des.Text = rfid_jiami;
            }
            catch (Exception exception)
            {
                MessageBox.Show($"加密异常：Uii_str={Uii_str};key_data={key_data};cmd_data={cmd_data};");
            }
        }

        private void button_lock_cmd_Click(object sender, EventArgs e)
        {
            string Uii_str = textBox_UII.Text;
            string rfid_cmd = RfidDll.rfid_lock_cmd(Uii_str);
            textBox_cmd.Text = rfid_cmd;
        }

        private void button_cmd_close_Click(object sender, EventArgs e)
        {
            string rfid_cmd = RfidDll.rfid_lock_cmd_close();
            textBox_cmd.Text = rfid_cmd;
        }

        private void button_cmd_open_Click(object sender, EventArgs e)
        {
            string rfid_cmd = RfidDll.rfid_lock_cmd_open();
            textBox_cmd.Text = rfid_cmd;
        }

        private void button_cmd_recover_Click(object sender, EventArgs e)
        {
            string rfid_cmd = RfidDll.rfid_lock_cmd_recover();
            textBox_cmd.Text = rfid_cmd;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            cb_SerialPort.Items.AddRange(SerialPort.GetPortNames());
            if (cb_SerialPort.Items.Count > 0)
            {
                cb_SerialPort.SelectedIndex = cb_SerialPort.Items.Count - 1;
            }

            textBox_UII.Text = "";
            textBox_key.Text = "";
            textBox_cmd.Text = "";
            button_close.Enabled = false;
            button_open.Enabled = false;
            button_rev.Enabled = false;
            button_read.Enabled = false;

            groupBox1.Enabled = false;
            cb_PALevel.SelectedIndex = 3;
        }

        private void btn_Open_Click(object sender, EventArgs e)
        {
            try
            {
                if (rb_Serial.Checked)
                {
                    if (cb_SerialPort.Text.Length > 0)
                    {
                        reader = new Reader(cb_SerialPort.Text);
                    }
                    else
                    {
                        MessageBox.Show("读选择串口！");
                    }
                }
                else
                {
                    reader = new Reader(tb_IP.Text, (int)nud_NetPort.Value);
                }
                if (reader.Open())
                {
                    reader.OnInventory += new Reader.InvDeleg(OnRadioInventory);
                    var fw = reader.GetFWVerion();
                    lb_status.Text = "已连接";
                    lb_status.ForeColor = Color.Blue;
                    this.Text += string.Format("  固件:{0}", fw);

                    deviceStatus = DeviceStatus.已连接;


                    groupBox2.Enabled = false;
                    groupBox1.Enabled = true;

                    button_close.Enabled = true;
                    button_open.Enabled = true;
                    button_rev.Enabled = true;
                    button_read.Enabled = true;

                    reader.InventoryStop();
                    reader.SetPALevel(1);
                    reader.SetTxPower(0, true, 24);

                    MessageBox.Show("打开成功");


                    return;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            MessageBox.Show("打开失败");
        }

        private void OnRadioInventory(TagInfo args)
        {
            if (!epcList.Contains(args.EPC))
            {
                epcList.Add(args.EPC);
                // todo 推送读取到的epc OnSendMsg(used_client, InitializeCmdType.InventoryInfo, args.EPC);
            }
        }

        /// <summary>
        /// 读标签
        /// </summary>
        private bool ReadEpc()
        {
            if (deviceStatus == DeviceStatus.已连接)
            {
                {
                    //先停止盘点，否则会卡死。
                    reader.InventoryStop();

                    //单次盘点，确认无其他标签
                    var tmpList = new List<TagInfo>();
                    for (int i = 0; i < 4; i++)
                    {
                        reader.InventorySingle(50).ForEach(s =>
                        {
                            if (!tmpList.Any(t => t.EPC == s.EPC))
                            {
                                tmpList.Add(s);
                            }
                        });
                    }

                    if (tmpList.Count > 1)
                    {
                        //绑定失败，存在其他标签
                        lb_status.Text = "操作失败，存在其他标签";
                    }
                    else if (tmpList.Count == 0)
                    {
                        //绑定失败，标签未放置
                        lb_status.Text = "操作失败，未读取到标签";
                    }
                    else
                    {
                        var uiiData = new OperData
                        {
                            EPC = "",
                            DATA = "",
                            Offset = 1,
                            Length = 8,
                            Bank = BankType.EPC,
                            Access = "00000000"
                        };
                        reader.ReadTag(ref uiiData);
                        textBox_UII.Text = uiiData.DATA;
                        var tidData = new OperData
                        {
                            EPC = "",
                            DATA = "",
                            Offset = 0,
                            Length = 6,
                            Bank = BankType.TID,
                            Access = "00000000"
                        };
                        reader.ReadTag(ref tidData);
                        textBox_key.Text = tidData.DATA;
                        if (string.IsNullOrEmpty(textBox_UII.Text) || string.IsNullOrEmpty(textBox_key.Text))
                        {
                            lb_status.Text = "操作失败，标签读取失败";
                            return false;
                        }

                        return true;
                    }
                }
            }
            else
            {
                deviceStatus = DeviceStatus.未连接;
                //设备未连接
            }

            return false;
        }

        /// <summary>
        /// 写标签
        /// </summary>
        private bool InitializeEpc(string data)
        {
            if (deviceStatus == DeviceStatus.已连接)
            {
                {
                    //将EPC长度改成位字符串
                    //48/6400
                    //24/3400
                    //64/8000

                    //先停止盘点，否则会卡死。
                    reader.InventoryStop();

                    OperData _oper = new OperData
                    {
                        EPC = "",
                        DATA = data,
                        Offset = 0,
                        Length = 20,
                        Bank = BankType.USER,
                        Access = "00000000"
                    };

                    var res = reader.WriteTag(_oper);
                    if (res == OperResult.OK)
                    {
                        //成功
                        lb_status.Text = "指令发送成功";
                        return true;
                    }
                    else
                    {
                        //绑定失败，写入异常
                        lb_status.Text = "指令发送失败，写入异常";
                    }
                }
            }
            else
            {
                deviceStatus = DeviceStatus.未连接;
                //设备未连接
            }
            return false;
        }

        private void button_close_Click(object sender, EventArgs e)
        {
            if (ReadEpc())
            {
                //关锁指令生成
                button_cmd_close_Click(null, null);
                //加密指令生成
                button_3des_Click(null, null);
                Application.DoEvents();
                //发送指令
                InitializeEpc(textBox_3des.Text);
                lb_lockStatus.Text = "锁状态：等待更新";
                Delay(2000);
                //重新读取状态
                button_read_Click(null, null);
            }

        }

        private void button_open_Click(object sender, EventArgs e)
        {
            if (ReadEpc())
            {
                //开锁指令生成
                button_cmd_open_Click(null, null);
                //加密指令生成
                button_3des_Click(null, null);
                Application.DoEvents();
                //发送指令
                InitializeEpc(textBox_3des.Text);
                lb_lockStatus.Text = "锁状态：等待更新";
                Delay(2000);
                //重新读取状态
                button_read_Click(null, null);
            }
        }

        private void button_rev_Click(object sender, EventArgs e)
        {
            if (ReadEpc())
            {
                //恢复指令生成
                button_cmd_recover_Click(null, null);
                //加密指令生成
                button_3des_Click(null, null);
                Application.DoEvents();
                //发送指令
                InitializeEpc(textBox_3des.Text);
                lb_lockStatus.Text = "锁状态：等待更新";
                Delay(2000);
                //重新读取状态
                button_read_Click(null, null);
            }
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (reader != null && reader.IsOpen())
            {
                reader.InventoryStop();
                reader.Close();
            }
        }

        private void button_read_Click(object sender, EventArgs e)
        {
            ReadEpc();
            if (!string.IsNullOrEmpty(textBox_UII.Text) && textBox_UII.Text.Length == 32)
            {
                var status = textBox_UII.Text.Substring(26, 2);
                switch (status)
                {
                    case "00":
                        {
                            lb_lockStatus.Text = "锁状态：开锁";
                            break;
                        }
                    case "01":
                        {
                            lb_lockStatus.Text = "锁状态：闭锁";
                            break;
                        }
                    case "06":
                        {
                            lb_lockStatus.Text = "锁状态：异常";
                            break;
                        }
                    default:
                        {
                            lb_lockStatus.Text = "锁状态：错误";
                            break;
                        }
                }
            }
            else
            {
                lb_lockStatus.Text = "锁状态：未知";
            }
        }

        public static void Delay(int milliSecond)
        {
            int start = Environment.TickCount;
            while (Math.Abs(Environment.TickCount - start) < milliSecond)
            {
                Application.DoEvents();
            }
        }

        private void btn_PowerSet_Click(object sender, EventArgs e)
        {
            if (reader != null && reader.IsOpen())
            {
                string errInfo = "";
                if (!(tb_PortEnable.Text.Length == 1 || tb_PortEnable.Text.Length == 4 || tb_PortEnable.Text.Length == 16))
                {
                    MessageBox.Show("天线数量不正确!");
                    return;
                }
                for (int i = 0; i < tb_PortEnable.Text.Length; i++)
                {
                    if (tb_PortEnable.Text[i] == '2')
                        continue;

                    bool enable = (tb_PortEnable.Text[i] == '1');
                    if (reader.SetTxPower((byte)i, enable, (double)nud_Power.Value) != OperResult.OK)
                    {
                        errInfo += "[" + i.ToString() + "]";
                    }
                }

                if (errInfo.Length > 0)
                {
                    MessageBox.Show("天线" + errInfo + "功率设置失败！");
                }
                else
                {
                    MessageBox.Show("功率设置成功!");
                }
            }
            else
            {
                MessageBox.Show("读写器未打开!");
            }
        }

        private void btn_PALevelSet_Click(object sender, EventArgs e)
        {
            if (reader != null && reader.IsOpen())
            {
                if (reader.SetPALevel((byte)cb_PALevel.SelectedIndex) == OperResult.OK)
                {
                    MessageBox.Show("功放等级操作成功!");
                }
                else
                {
                    MessageBox.Show("功放等级操作失败!");
                }
            }
            else
            {
                MessageBox.Show("读写器未打开!");
            }
        }

        private void Form1_KeyDown(object sender, KeyEventArgs e)
        {
            if (button_close.Enabled == false)
            {
                //未开启，过滤
                return;
            }
            switch (e.KeyCode)
            {
                case Keys.F1:
                    button_close_Click(this, EventArgs.Empty);
                    break;
                case Keys.F2:
                    button_open_Click(this, EventArgs.Empty);
                    break;
                case Keys.F3:
                    button_rev_Click(this, EventArgs.Empty);
                    break;
            }
        }
    }
    public enum DeviceStatus
    {
        未连接,
        已连接
    }
}
